float swt = 25.0;     //sep.mult(25.0f);
float awt = 4.0;      //ali.mult(4.0f);
float cwt = 5.0;      //coh.mult(5.0f);
float maxspeed = 1;
float maxforce = 0.025;


// Flocking
// Daniel Shiffman <http://www.shiffman.net>
// The Nature of Code, Spring 2009

// Boid class
// Methods for Separation, Cohesion, Alignment added

class Boid {

  PVector loc;
  PVector vel;
  PVector acc;
  PVector[] segment; 
  float tailleSegment = 20;
  int nombreSegment;
  float r;
  int typeMonstre; // 0 : Lombric, 1 : Mouche

  Boid(float x, float y, String _typeMonstre) {
    acc = new PVector(0, 0);
    vel = new PVector(random(-1, 1), random(-1, 1));
    loc = new PVector(x, y);
    if (_typeMonstre.equals("Lombric")) {
      typeMonstre = 0;
      nombreSegment = 5;
    }
    if (_typeMonstre.equals("Mouche")) {
      typeMonstre = 1;
      nombreSegment = 2;
    }
    r = 10.0;
    segment = new PVector[nombreSegment];
    for (int i = 0; i < segment.length; i++) {
      segment[i] = new PVector(0, 0);
    }
  }

  void run(ArrayList<Boid> boids, Blob[] blobs) {
    flock(boids, blobs);
    update();
    //borders();
    render();
  }

  void applyForce(PVector force) {
    // We could add mass here if we want A = F / M
    acc.add(force);
  }

  // We accumulate a new acceleration each time based on three rules
  void flock(ArrayList<Boid> boids, Blob[] blobs) {
    PVector sep = separate(boids);   // Separation
    PVector ali = align(boids);      // Alignment
    PVector coh = cohesion(boids);   // Cohesion
    PVector bor = bordure();
    PVector attraction = fuite(blobs);
    // Arbitrarily weight these forces
    sep.mult(swt);
    ali.mult(awt);
    coh.mult(cwt);
    //bor.mult(1.0);
    attraction.mult(-20);
    // Add the force vectors to acceleration
    applyForce(sep);
    applyForce(attraction);
    applyForce(bor);
    if (typeMonstre == 0) {
      applyForce(ali);
      applyForce(coh);
    }
  }

  // Method to update location
  void update() {
    // Update velocity
    vel.add(acc);
    // Limit speed
    vel.limit(maxspeed);
    loc.add(vel);
    // Reset accelertion to 0 each cycle
    acc.mult(0);
  }

  // A method that calculates and applies a steering force towards a target
  // STEER = DESIRED MINUS VELOCITY
  PVector seek(PVector target) {
    PVector desired = PVector.sub(target, loc);  // A vector pointing from the location to the target

    // Normalize desired and scale to maximum speed
    desired.normalize();
    desired.mult(maxspeed);
    // Steering = Desired minus Velocity
    PVector steer = PVector.sub(desired, vel);
    steer.limit(maxforce);  // Limit to maximum steering force
    return steer;
  }

  void render() {
    // Draw a triangle rotated in the direction of velocity
    float theta = vel.heading2D() + radians(90);
    //strokeWeight(5);
    noStroke();
    fill(0);
    dragSegment(0, loc);
    for (int i=0; i<segment.length-1; i++) {
      dragSegment(i+1, segment[i]);
    }
  }

  void dragSegment(int i, PVector segmentInt) {
    PVector delta = PVector.sub(segmentInt, segment[i]);
    float angle = atan2(delta.y, delta.x);  
    segment[i].x = segmentInt.x - cos(angle) * tailleSegment;
    segment[i].y = segmentInt.y - sin(angle) * tailleSegment;
    switch (typeMonstre) {
    case 0 :
      segmentLombric(segment[i], angle, i);
      break;
    case 1 : 
      segmentMouche(segment[i], angle, i);
      break;
    }
  }

  void segmentLombric(PVector seg, float a, int numero) {
    pushMatrix();
    translate(seg.x, seg.y);
    rotate(a);
    if (radio) {
      noFill();
      stroke(100, 255, 0);
    } 
    else {
      fill(0);
      noStroke();
    }
    ellipse(tailleSegment/2, 0, tailleSegment, tailleSegment/(numero + 1));
    if (numero == 0) {
      if (radio) {
        noFill();
      } 
      else {
        fill(255);
      }
      ellipse(3*tailleSegment/5, 0, tailleSegment/2, tailleSegment/2);
      if (radio) {
        noFill();
      } 
      else {
        fill(0);
      }
      ellipse(3*tailleSegment/4, 0, tailleSegment/5, tailleSegment/5);
    }
    popMatrix();
  }

  void segmentMouche(PVector seg, float a, int numero) {
    pushMatrix();
    translate(seg.x, seg.y);
    rotate(a);
    switch(numero) {
    case 0 : 
      if (radio) {
        noFill();
        stroke(100, 255, 0);
      } 
      else {
        fill(0);
        noStroke();
      }
      //Segment 1
      //Tête
      ellipse(tailleSegment/3, 0, 2*tailleSegment/3 + 1, tailleSegment/ 1.2);
      //Yeux
      ellipse(tailleSegment/3, - tailleSegment/ 2.9, tailleSegment / 2, tailleSegment / 2);
      ellipse(tailleSegment/3, tailleSegment/ 2.9, tailleSegment / 2, tailleSegment / 2);

      if (radio) {
        noFill();
        stroke(100, 255, 0);
        strokeWeight(1);
      } 
      else {
        stroke(0);
        strokeWeight(tailleSegment/10);
      }      
      line(tailleSegment, 0, tailleSegment/2, 0);

      if (radio) {
        noFill();
        stroke(100, 255, 0);
      } 
      else {
        noStroke();
        fill(255);
      }
      ellipse(tailleSegment/3, - tailleSegment/ 2.9, tailleSegment / 3, tailleSegment / 3);
      ellipse(tailleSegment/3, tailleSegment/ 2.9, tailleSegment / 3, tailleSegment / 3);
      if (radio) {
        noFill();
        stroke(100, 255, 0);
      } 
      else {
        fill(0);
        noStroke();
      }
      ellipse(tailleSegment/3, - tailleSegment/ 2.9, tailleSegment / 6, tailleSegment / 3);
      ellipse(tailleSegment/3, tailleSegment/ 2.9, tailleSegment / 6, tailleSegment / 3);
      break;
    case 1 :
      if (radio) {
        noFill();
        stroke(100, 255, 0);
        strokeWeight(tailleSegment / 20);
      } 
      else {
        fill(255);
        stroke(0);
        strokeWeight(tailleSegment / 20);
      }
      bezier(0, 0, tailleSegment/8, 1.2*tailleSegment, 2*tailleSegment, tailleSegment, tailleSegment/2, 0); 
      bezier(0, 0, tailleSegment/8, -1.2*tailleSegment, 2*tailleSegment, -tailleSegment, tailleSegment/2, 0); 
      if (radio) {
        noFill();
        stroke(100, 255, 0);
      } 
      else {
        fill(0);
        noStroke();
      }
      ellipse(tailleSegment /2, 0, 1.2 * tailleSegment, tailleSegment/1.8);
      break;
    }
    popMatrix();
  }

  PVector bordure() {
    float coefficient = 1.15;
    PVector effetBordure = new PVector(0, 0, 0);
    if (loc.x < 80) {
      effetBordure.add(new PVector(pow(coefficient, 10 - loc.x), 0));
    }
    if (loc.x > width - 40) {
      effetBordure.add(new PVector(-pow(coefficient, 10 - (width - loc.x)), 0));
    }
    if (loc.y < 40) {
      effetBordure.add(new PVector(0, pow(coefficient, 10 - loc.y)));
    }
    if (loc.y > height - 40) {
      effetBordure.add(new PVector(0, -pow(coefficient, 10 - (height - loc.y))));
    }
    //effetBordure.div(10);
    return effetBordure;
  }

  // Wraparound
  void borders() {
    if (loc.x < 0) vel.x = -vel.x;
    if (loc.y < 0) vel.y = -vel.y;
    if (loc.x > width) vel.x = -vel.x;
    if (loc.y > height) vel.y = -vel.y;
  }

  // Separation
  // Method checks for nearby boids and steers away
  PVector separate (ArrayList<Boid> boids) {
    float desiredseparation = 25.0;
    PVector steer = new PVector(0, 0);
    int count = 0;
    // For every boid in the system, check if it's too close
    for (Boid other : boids) {
      float d = PVector.dist(loc, other.loc);
      // If the distance is greater than 0 and less than an arbitrary amount (0 when you are yourself)
      if ((d > 0) && (d < desiredseparation)) {
        // Calculate vector pointing away from neighbor
        PVector diff = PVector.sub(loc, other.loc);
        diff.normalize();
        diff.div(d);        // Weight by distance
        steer.add(diff);
        count++;            // Keep track of how many
      }
    }
    // Average -- divide by how many
    if (count > 0) {
      steer.div((float)count);
      // Implement Reynolds: Steering = Desired - Velocity
      steer.normalize();
      steer.mult(maxspeed);
      steer.sub(vel);
      steer.limit(maxforce);
    }
    return steer;
  }

  // Alignment
  // For every nearby boid in the system, calculate the average velocity
  PVector align (ArrayList<Boid> boids) {
    float neighbordist = 50.0;
    PVector steer = new PVector();
    int count = 0;
    for (Boid other : boids) {
      float d = PVector.dist(loc, other.loc);
      if ((d > 0) && (d < neighbordist)) {
        steer.add(other.vel);
        count++;
      }
    }
    if (count > 0) {
      steer.div((float)count);
      // Implement Reynolds: Steering = Desired - Velocity
      steer.normalize();
      steer.mult(maxspeed);
      steer.sub(vel);
      steer.limit(maxforce);
    }
    return steer;
  }

  // Cohesion
  // For the average location (i.e. center) of all nearby boids, calculate steering vector towards that location
  PVector cohesion (ArrayList<Boid> boids) {
    float neighbordist = 50.0;
    PVector sum = new PVector(0, 0);   // Start with empty vector to accumulate all locations
    int count = 0;
    for (Boid other : boids) {
      float d = PVector.dist(loc, other.loc);
      if ((d > 0) && (d < neighbordist)) {
        sum.add(other.loc); // Add location
        count++;
      }
    }
    if (count > 0) {
      sum.div((float)count);
      return seek(sum);  // Steer towards the location
    }
    return sum;
  }
  
  PVector fuite(Blob[] blobs) {
    PVector somme = new PVector(0, 0, 0);
    int compteur = 0;
    for (int i = 0; i < blobs.length; i++ ) {
      float diametre = blobs[i].length;
      PVector centreBlob = new PVector(blobs[i].centroid.x, blobs[i].centroid.y);
      float d = PVector.dist(loc, centreBlob);
      if ((d > 0) && (d < diametre/3)) {
        somme.add(centreBlob);
        compteur++;
      }
    }
    if (compteur > 0) {
      somme.div((float)compteur);
      return seek(somme);
    }
    return somme;
  }
}

