
// x pour afficher les valeurs + boid
// a pour augmenter le seuil
// q pour baisser le seuil
// p pour décaler les boids vers le haut
// m pour décaler les boids vers le bas

import processing.opengl.*;
import org.openkinect.*;
import org.openkinect.processing.*;
import hypermedia.video.*;
import java.awt.*;
import processing.serial.*;

OpenCV opencv;
Kinect kinect;
Flock flock;
Serial portSerie;

boolean showvalues = false;
boolean scrollbar = false;
boolean radio;

// Kinect
int largeurKinect = 640;
int hauteurKinect = 480;
int seuil = 690; // Distance en millimètre de la Kinect 4500 à bourges
int Yvect = 100;
PImage calcul;
float deg = 15; // Start at 15 degrees
int tailleMaxRecherche;
int nombreMaxBlobs;
int tampon; // Tampon des blobs
int[][] moyenne;
int temoin;
Blob[] blobs;
// int compteur;
PVector correctionKinect;
// PVector correctionCentreBlobs;

void setup() {
  size(1024, 768, OPENGL);
  frameRate(60);
  smooth(4); // Réglage de l'anti aliasing 0, 2, 4 ou 8

  // Kinect
  kinect = new Kinect(this);
  kinect.start();
  kinect.enableDepth(true);
  calcul = createImage(largeurKinect, hauteurKinect, RGB);
  calcul.loadPixels();
  correctionKinect = new PVector(0, Yvect); // Vecteur de correction pour compenser le décalage entre la kinect et le vidéoprojecteur (ajout systématique de valeurs en x et y aux blobs calculés)

  // Blob
  opencv = new OpenCV(this);
  opencv.allocate(largeurKinect, hauteurKinect);
  tailleMaxRecherche = largeurKinect * hauteurKinect / 3;
  nombreMaxBlobs = 10; // Nbre max de blobs (personnes) détectées simultanément
  tampon = 10;
  moyenne = new int[nombreMaxBlobs][tampon];

  //Aquarium
  setupScrollbars();
  colorMode(RGB, 255, 255, 255, 100);
  flock = new Flock();
  // Initialisation des boids
  for (int i = 0; i < 20; i++) {
    flock.addBoid(new Boid(width/2, height/2, "Lombric"));
  }
  for (int i = 0; i < 20; i++) {
    flock.addBoid(new Boid(width/2, height/2, "Mouche"));
  }

  // Arduino
  for (int i = 0; i < Serial.list().length; i++) {
    println(Serial.list()[i]);
  }
  String nomPort = Serial.list()[0];
  portSerie = new Serial(this, nomPort, 115200);

  // Mode radioActif off
  radio = false;

  // Préparation pour plein ecran 
  //frame.removeNotify();
  //frame.setUndecorated(true);
  //frame.addNotify();
}

void draw() {
  //  if(frameCount == 1) {
  //      //frame.setLocation(1363, -28);
  //      //frame.setLocation(, 0);
  //}


  // Arduino
  if (portSerie.available() > 0) { 
    portSerie.read();
    radio = true;
    temoin = frameCount;
  }

  // Raffraichissement écran
  if ((radio) && ((frameCount - temoin) < 5)) {
    background(0);
  } 
  else {
    background(255, 255, 255);
    radio = false;
  }

  // Kinect
  int[] profondeur = kinect.getRawDepth();
  for (int i = 0; i < profondeur.length; i++) {
    // for (int i = 0; i < largeurKinect * hauteurKinect; i++) {
    calcul.pixels[i] =  (profondeur[i] < seuil) ? 0xFFFFFF : 0x000000;
  }
  calcul.updatePixels();
  image( calcul, 0, 0, width, height);

  // Blobs
  opencv.copy(calcul);
  blobs = opencv.blobs(tailleMaxRecherche/200, tailleMaxRecherche, 10, false );
  for ( int i=0; i < blobs.length; i++ ) {
    float diametre = blobs[i].length;
    blobs[i].centroid.x = blobs[i].centroid.x * width / largeurKinect + (int)correctionKinect.x;
    blobs[i].centroid.y = blobs[i].centroid.y * height / hauteurKinect + (int)correctionKinect.y;
    if (showvalues) {
      noFill();
      strokeWeight(3);
      stroke(255, 0, 0);
      ellipse(blobs[i].centroid.x, blobs[i].centroid.y, diametre/3, diametre/3);
    }
  }

  if (blobs != null) {
    flock.run(blobs);
  }

  // Aquarium
  drawScrollbars();

  if (mousePressed && !scrollbar) {
    switch (round(random(0, 2))) {
    case 0 :
      flock.addBoid(new Boid(mouseX, mouseY, "Lombric"));
      break;
    case 1 :
      flock.addBoid(new Boid(mouseX, mouseY, "Mouche"));
      break;
    }
  }

  if (showvalues) {
    fill(255, 0, 0);
    textAlign(LEFT);
    text("Total boids: " + flock.boids.size() + "\n" + "Framerate: " + round(frameRate) + "\nPress any key to show/hide sliders and text\nClick mouse to add more boids" + "\nSeuil: " + seuil + "\nVecteur-Y: " + Yvect, 5, 100);
  }
}

void keyPressed() {
  if(key == 'x') showvalues = !showvalues;
  else if(key == 'a') seuil++;
  else if(key == 'q') seuil--;
  else if(key == 'p') Yvect--;
  else if(key == 'm') Yvect++;
  correctionKinect = new PVector(0, Yvect);
  

}

void mousePressed() {
}

void stop() {
  kinect.quit();
  opencv.stop();
  super.stop();
}

